# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from increase import Increase, AsyncIncrease
from tests.utils import assert_matches_type
from increase.types import (
    PhysicalCardProfile,
)
from increase.pagination import SyncPage, AsyncPage

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestPhysicalCardProfiles:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: Increase) -> None:
        physical_card_profile = client.physical_card_profiles.create(
            carrier_image_file_id="file_h6v7mtipe119os47ehlu",
            contact_phone="+16505046304",
            description="My Card Profile",
            front_image_file_id="file_o6aex13wm1jcc36sgcj1",
        )
        assert_matches_type(PhysicalCardProfile, physical_card_profile, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: Increase) -> None:
        response = client.physical_card_profiles.with_raw_response.create(
            carrier_image_file_id="file_h6v7mtipe119os47ehlu",
            contact_phone="+16505046304",
            description="My Card Profile",
            front_image_file_id="file_o6aex13wm1jcc36sgcj1",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        physical_card_profile = response.parse()
        assert_matches_type(PhysicalCardProfile, physical_card_profile, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: Increase) -> None:
        with client.physical_card_profiles.with_streaming_response.create(
            carrier_image_file_id="file_h6v7mtipe119os47ehlu",
            contact_phone="+16505046304",
            description="My Card Profile",
            front_image_file_id="file_o6aex13wm1jcc36sgcj1",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            physical_card_profile = response.parse()
            assert_matches_type(PhysicalCardProfile, physical_card_profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_retrieve(self, client: Increase) -> None:
        physical_card_profile = client.physical_card_profiles.retrieve(
            "string",
        )
        assert_matches_type(PhysicalCardProfile, physical_card_profile, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: Increase) -> None:
        response = client.physical_card_profiles.with_raw_response.retrieve(
            "string",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        physical_card_profile = response.parse()
        assert_matches_type(PhysicalCardProfile, physical_card_profile, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: Increase) -> None:
        with client.physical_card_profiles.with_streaming_response.retrieve(
            "string",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            physical_card_profile = response.parse()
            assert_matches_type(PhysicalCardProfile, physical_card_profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: Increase) -> None:
        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `physical_card_profile_id` but received ''"
        ):
            client.physical_card_profiles.with_raw_response.retrieve(
                "",
            )

    @parametrize
    def test_method_list(self, client: Increase) -> None:
        physical_card_profile = client.physical_card_profiles.list()
        assert_matches_type(SyncPage[PhysicalCardProfile], physical_card_profile, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: Increase) -> None:
        physical_card_profile = client.physical_card_profiles.list(
            cursor="string",
            idempotency_key="x",
            limit=1,
            status={"in": ["pending_creating", "pending_reviewing", "rejected"]},
        )
        assert_matches_type(SyncPage[PhysicalCardProfile], physical_card_profile, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: Increase) -> None:
        response = client.physical_card_profiles.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        physical_card_profile = response.parse()
        assert_matches_type(SyncPage[PhysicalCardProfile], physical_card_profile, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: Increase) -> None:
        with client.physical_card_profiles.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            physical_card_profile = response.parse()
            assert_matches_type(SyncPage[PhysicalCardProfile], physical_card_profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_archive(self, client: Increase) -> None:
        physical_card_profile = client.physical_card_profiles.archive(
            "string",
        )
        assert_matches_type(PhysicalCardProfile, physical_card_profile, path=["response"])

    @parametrize
    def test_raw_response_archive(self, client: Increase) -> None:
        response = client.physical_card_profiles.with_raw_response.archive(
            "string",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        physical_card_profile = response.parse()
        assert_matches_type(PhysicalCardProfile, physical_card_profile, path=["response"])

    @parametrize
    def test_streaming_response_archive(self, client: Increase) -> None:
        with client.physical_card_profiles.with_streaming_response.archive(
            "string",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            physical_card_profile = response.parse()
            assert_matches_type(PhysicalCardProfile, physical_card_profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_archive(self, client: Increase) -> None:
        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `physical_card_profile_id` but received ''"
        ):
            client.physical_card_profiles.with_raw_response.archive(
                "",
            )

    @parametrize
    def test_method_clone(self, client: Increase) -> None:
        physical_card_profile = client.physical_card_profiles.clone(
            "string",
        )
        assert_matches_type(PhysicalCardProfile, physical_card_profile, path=["response"])

    @parametrize
    def test_method_clone_with_all_params(self, client: Increase) -> None:
        physical_card_profile = client.physical_card_profiles.clone(
            "string",
            carrier_image_file_id="string",
            contact_phone="x",
            description="x",
            front_image_file_id="file_o6aex13wm1jcc36sgcj1",
            front_text={
                "line1": "x",
                "line2": "x",
            },
        )
        assert_matches_type(PhysicalCardProfile, physical_card_profile, path=["response"])

    @parametrize
    def test_raw_response_clone(self, client: Increase) -> None:
        response = client.physical_card_profiles.with_raw_response.clone(
            "string",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        physical_card_profile = response.parse()
        assert_matches_type(PhysicalCardProfile, physical_card_profile, path=["response"])

    @parametrize
    def test_streaming_response_clone(self, client: Increase) -> None:
        with client.physical_card_profiles.with_streaming_response.clone(
            "string",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            physical_card_profile = response.parse()
            assert_matches_type(PhysicalCardProfile, physical_card_profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_clone(self, client: Increase) -> None:
        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `physical_card_profile_id` but received ''"
        ):
            client.physical_card_profiles.with_raw_response.clone(
                "",
            )


class TestAsyncPhysicalCardProfiles:
    parametrize = pytest.mark.parametrize("async_client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    async def test_method_create(self, async_client: AsyncIncrease) -> None:
        physical_card_profile = await async_client.physical_card_profiles.create(
            carrier_image_file_id="file_h6v7mtipe119os47ehlu",
            contact_phone="+16505046304",
            description="My Card Profile",
            front_image_file_id="file_o6aex13wm1jcc36sgcj1",
        )
        assert_matches_type(PhysicalCardProfile, physical_card_profile, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncIncrease) -> None:
        response = await async_client.physical_card_profiles.with_raw_response.create(
            carrier_image_file_id="file_h6v7mtipe119os47ehlu",
            contact_phone="+16505046304",
            description="My Card Profile",
            front_image_file_id="file_o6aex13wm1jcc36sgcj1",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        physical_card_profile = response.parse()
        assert_matches_type(PhysicalCardProfile, physical_card_profile, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncIncrease) -> None:
        async with async_client.physical_card_profiles.with_streaming_response.create(
            carrier_image_file_id="file_h6v7mtipe119os47ehlu",
            contact_phone="+16505046304",
            description="My Card Profile",
            front_image_file_id="file_o6aex13wm1jcc36sgcj1",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            physical_card_profile = await response.parse()
            assert_matches_type(PhysicalCardProfile, physical_card_profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncIncrease) -> None:
        physical_card_profile = await async_client.physical_card_profiles.retrieve(
            "string",
        )
        assert_matches_type(PhysicalCardProfile, physical_card_profile, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncIncrease) -> None:
        response = await async_client.physical_card_profiles.with_raw_response.retrieve(
            "string",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        physical_card_profile = response.parse()
        assert_matches_type(PhysicalCardProfile, physical_card_profile, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncIncrease) -> None:
        async with async_client.physical_card_profiles.with_streaming_response.retrieve(
            "string",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            physical_card_profile = await response.parse()
            assert_matches_type(PhysicalCardProfile, physical_card_profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncIncrease) -> None:
        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `physical_card_profile_id` but received ''"
        ):
            await async_client.physical_card_profiles.with_raw_response.retrieve(
                "",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncIncrease) -> None:
        physical_card_profile = await async_client.physical_card_profiles.list()
        assert_matches_type(AsyncPage[PhysicalCardProfile], physical_card_profile, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncIncrease) -> None:
        physical_card_profile = await async_client.physical_card_profiles.list(
            cursor="string",
            idempotency_key="x",
            limit=1,
            status={"in": ["pending_creating", "pending_reviewing", "rejected"]},
        )
        assert_matches_type(AsyncPage[PhysicalCardProfile], physical_card_profile, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncIncrease) -> None:
        response = await async_client.physical_card_profiles.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        physical_card_profile = response.parse()
        assert_matches_type(AsyncPage[PhysicalCardProfile], physical_card_profile, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncIncrease) -> None:
        async with async_client.physical_card_profiles.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            physical_card_profile = await response.parse()
            assert_matches_type(AsyncPage[PhysicalCardProfile], physical_card_profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_archive(self, async_client: AsyncIncrease) -> None:
        physical_card_profile = await async_client.physical_card_profiles.archive(
            "string",
        )
        assert_matches_type(PhysicalCardProfile, physical_card_profile, path=["response"])

    @parametrize
    async def test_raw_response_archive(self, async_client: AsyncIncrease) -> None:
        response = await async_client.physical_card_profiles.with_raw_response.archive(
            "string",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        physical_card_profile = response.parse()
        assert_matches_type(PhysicalCardProfile, physical_card_profile, path=["response"])

    @parametrize
    async def test_streaming_response_archive(self, async_client: AsyncIncrease) -> None:
        async with async_client.physical_card_profiles.with_streaming_response.archive(
            "string",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            physical_card_profile = await response.parse()
            assert_matches_type(PhysicalCardProfile, physical_card_profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_archive(self, async_client: AsyncIncrease) -> None:
        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `physical_card_profile_id` but received ''"
        ):
            await async_client.physical_card_profiles.with_raw_response.archive(
                "",
            )

    @parametrize
    async def test_method_clone(self, async_client: AsyncIncrease) -> None:
        physical_card_profile = await async_client.physical_card_profiles.clone(
            "string",
        )
        assert_matches_type(PhysicalCardProfile, physical_card_profile, path=["response"])

    @parametrize
    async def test_method_clone_with_all_params(self, async_client: AsyncIncrease) -> None:
        physical_card_profile = await async_client.physical_card_profiles.clone(
            "string",
            carrier_image_file_id="string",
            contact_phone="x",
            description="x",
            front_image_file_id="file_o6aex13wm1jcc36sgcj1",
            front_text={
                "line1": "x",
                "line2": "x",
            },
        )
        assert_matches_type(PhysicalCardProfile, physical_card_profile, path=["response"])

    @parametrize
    async def test_raw_response_clone(self, async_client: AsyncIncrease) -> None:
        response = await async_client.physical_card_profiles.with_raw_response.clone(
            "string",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        physical_card_profile = response.parse()
        assert_matches_type(PhysicalCardProfile, physical_card_profile, path=["response"])

    @parametrize
    async def test_streaming_response_clone(self, async_client: AsyncIncrease) -> None:
        async with async_client.physical_card_profiles.with_streaming_response.clone(
            "string",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            physical_card_profile = await response.parse()
            assert_matches_type(PhysicalCardProfile, physical_card_profile, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_clone(self, async_client: AsyncIncrease) -> None:
        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `physical_card_profile_id` but received ''"
        ):
            await async_client.physical_card_profiles.with_raw_response.clone(
                "",
            )
